@@ -17,6 +17,7 @@
 import os
 import sys
 import io
+import re
 import logging
 from builtins import open as _open
 from collections import deque, OrderedDict
@@ -32,55 +33,204 @@ import networkx as nx
 
 
 from Bio import bgzf
+from Bio.Seq import Seq
 from kmerdb import fileutil, parse, kmer, config, util
 
 
 logger = logging.getLogger(__file__)
 
 
-def graph_convert(n1:np.ndarray, n2:np.ndarray, weights:np.ndarray, k:int=None):
-    """
-    :param n1: A numpy.ndarray of node 1 kmer ids from an edge list
-    :type numpy.ndarray:
-    :param n2: A numpy.ndarray of node 2 kmer ids from an edge list
-    :type numpy.ndarray:
-    :param weights: A numpy.ndarray of weights corresponding to the edge
-    :type numpy.ndarray:
-    """
 
+def make_secondary_graph_from_edge_list(seq_id:str, k:int, edges:list):
+    """
+    Creates a list of k+1 mer edges from a list of 2-tuples of kmerids (the primary edge list)
+    :param seq_id: A sequence id for all corresponding edges
+    :type seq_id: str
+    :param edges: A list of 2-tuples of k-mer ids
+    :type edges: list
+    :raises TypeError: if seq_id is not a str
+    :raises TypeError: if k is not an int
+    :raises TypeError: if edges is not a list of N 2-tuples of kmer id int
+    :raises ValueError: if the number of new edges does not have length of N-1
+    :returns: the edge list of the k+1mer graph (each edge corresponds to a k+2mer)
+    :rtype: list
+    """
+    if type(seq_id) is not str:
+        raise TypeError("kmerdb.graph.make_secondary_graph_from_edge_list() expects a str as its first positional argument")
+    elif type(k) is not int:
+        raise TypeError("kmerdb.graph.make_secondary_graph_from_edge_list() expects an int as its second positional argument")
+    elif type(edges) is not list or not all(type(e) is tuple and type(e[0]) is int and type(e[1]) is int for e in edges):
+        raise TypeError("kmerdb.graph.make_secondary_graph_from_edge_list() expects a list of integer pairs as its third positional argument")
+
+    new_edges = []
+    kplus1mers = convert_edge_list_to_augmented_kmer(k, edges)
+    num_edges = len(kplus1mers)
+    for i, kmer1 in enumerate(kplus1mers):
+        if i < num_edges - 1:
+            kmer2 = augmented_kplus1mers[i+1]
+            new_edges.append( (kmer.kmer_to_id(kmer1), kmer.kmer_to_id(kmer2)) )
+    if len(new_edges) != num_edges - 1:
+        raise ValueError("Internal error: kmerdb.graph.make_secondary_graph_from_edge_list() should create N-1 k+1mers from an edges list with length N")
+    return new_edges
+
+def convert_edge_list_to_augmented_kmer(k:int, edges:list):
+    """
+    Creates a list of k+1mer strings from an edge list of 2-tuples of kmer ids
+    :param k:
+    :type k: int
+    :param edges:
+    :type edges: list
+    :raises TypeError: if k is not an int
+    :raises TypeError: if edges is not a list of 2-tuples of kmer id int
+    :returns: a list of k+1mer strings from the k-mer edge list
+    :rtype: list
+    """
     if type(k) is not int:
-        raise TypeError("kmerdb.graphlib.graph_convert() expects the keyword argument 'k' to be an int")
-    elif isinstance(n1, np.ndarray) is False:
-        raise TypeError("kmerdb.graphlib.graph_convert() expects a numpy.ndarray as its first positional argument")
-    elif isinstance(n2, np.ndarray) is False:
-        raise TypeError("kmerdb.graphlib.graph_convert() expects a numpy.ndarray as its second positional argument")
-    elif isinstance(weights, np.ndarray) is False:
-        raise TypeError("kmerdb.graphlib.graph_convert() expects a numpy.ndarray as its third positional argument")
-    elif n1.shape == () or n2.shape == () or weights.shape == ():
-        raise ValueError("kmerdb.graphlib.graph_convert() found that numpy.ndarrays were empty")
-    elif n1.shape != n2.shape:
-        raise ValueError("kmerdb.graphlib.graph_convert() found that the node1 and node2 array lengths did not match |  node1: {0} node2: {1}".format(n1.shape, n2.shape))
-    elif n2.shape != weights.shape:
-        raise ValueError("kmerdb.graphlib.graph_convert() found that the node2 and weights array lengths did not match |  node2: {0} weights: {1}".format(n2.shape, weights.shape))
+        raise TypeError("kmerdb.graph.convert_edge_list_to_augmented_kmer() expects an int as its first positional argument")
+    elif type(edges) is not list or not all(type(e) is tuple and type(e[0]) is int and type(e[1]) is int for e in edges):
+        raise TypeError("kmerdb.graph.convert_edge_list_to_augmented_kmer() expects a list of integer pairs as its third positional argument")
 
-    N = 4**k
-    # edge_list = list(zip(n1.tolist(), n2.tolist(), weights.tolist()))
+    augmented_kplus1mers = []
+    num_edges = len(edges)
+    for i, edge in edges:
+        kmerid1, kmerid2 = edge
+        kmer1 = kmer.id_to_kmer(kmerid1, k)
+        kmer2 = kmer.id_to_kmer(kmerid2, k)
+        if kmer.is_right_neighbor(kmer1, kmer2):
+            bridging_kmer = kmer1 + kmer2[-1]
+            augmented_kplus1mers.append(kmer.kmer_to_id(bridging_kmer))
+        else:
+            logger.error(f"Sequence {seq_id}  |   {kmer1} @ position {i}  -  {kmer2} @ position {i+1}")
+            raise ValueError(f"Internal error: lexicographically ordered kmers don't appear to be valid neighbors in the sequence.")
+    return augmented_kplus1mers
+
+
+def make_edges_from_kmerids(kmer_ids:list, positions:list, seqlen:int, seq_id:str, k:int):
+    """
+    Make edges from a list of lexicographically ordered kmer ids, given their positions and seq_ids
+    :param kmer_ids: A list of kmer ids as ints
+    :type kmer_ids: list
+    :param positions: A list of positions in the 'seq_id' sequence/read
+    :type positions: list
+    :param seqlen: The length of the sequence
+    :type seqlen: int
+    :param seq_id: A sequence identifier from the fasta/fastq sequence
+    :type seq_id: str
+    :param k:  The choice of k
+    :type k: int
+    :raises TypeError: if the kmer_ids positional arg is not a list of integers
+    :raises TypeError: if the positions positional arg is not a list of integers
+    :raises ValueError: if the list of kmer_ids and positions are not equal in length
+    :raises TypeError: if the sequence identifier is not a str
+    :raises TypeError: if the k param is not an int
+    :returns: A list of 5-tuples - (sequence_id, position1, kmerid1, position2, kmerid2)
+    :rtype: list
+    """
+    if type(kmer_ids) is not list and not all(type(kid) is int for kid in kmer_ids):
+        raise TypeError("kmerdb.graph.make_edges_from_kmerids() expects a list of kmerids as its first positional argument")
+    elif type(positions) is not list and not all(type(p) is int for p in positions):
+        raise TypeError("kmerdb.graph.make_edges_from_kmerids() expects a list of int positions in the sequence as its second positional argument")
+    elif len(kmer_ids) != len(positions):
+        raise ValueError("kmerdb.graph.make_edges_from_kmerids() expects the lists of kmer_ids and positions to be equal in length")
+    elif type(seqlen) is not int:
+        raise TypeError("kmerdb.graph.make_edges_from_kmerids() expects an int sequence length as its third positional argument")
+    elif type(seq_id) is not str:
+        raise TypeError("kmerdb.graph.make_edges_from_kmerids() expects a sequence identifier as a str as its fourth positional argument")
+    elif type(k) is not int:
+        raise TypeError("kmerdb.graph.make_edges_from_kmerids() expects an int as its fifth positional argument")
+
+    num_kmer_ids = len(kmer_ids)
+    final = []
+    for i in range(num_kmer_ids):
+        kmer_id = kmer_ids[i]
+        if kmer_id is not None:
+            current_kmer = kmer.id_to_kmer(kmer_id, k)
+            pos = positions[i]
+
+            if positions[i] == 0:
+                # If i = 0, the very first in the list, do nothing.
+                continue
+            elif pos + k + 1 == seqlen:
+                # If the position is at the end of a sequence, append the last k-mer pair
+                final.append( (seq_id, positions[i-1], kmer_ids[i-1], pos, kmer_id, ) ) # The final result is an edge tuple (sequence_id, position1, kmerid1, position2, kmerid2)
+            else:
+                """
+                Here, internal (not terminal) k-mer adjacencies/edges are produced from a list of kmer_ids
+                Local neighborhoods are created to correctly resolve k-mer adjacencies via lexical ordering while
+                accounting for the correct kmer-to-kmer adjacency wrt ambiguous residues
+                """
+                left_neighbors = [] # Create local neighbor lists in case of substitution of ambiguous residue
+                j = i - 1
+                tracking_left = True
+                while tracking_left is True:
+                    # print(f"index: {i}")
+                    # print(f"pos[j] : {positions[j]}")
+                    # print(f"current kmer : {current_kmer}")
+                    # print(f" left neighbor : {kmer.id_to_kmer(kmer_ids[j], k)}")
+                    # print(f"is left neighbor : {kmer.is_left_neighbor(current_kmer, kmer.id_to_kmer(kmer_ids[j], k))}")
+                    # print(list(map(lambda kid: kmer.id_to_kmer(kid, k), kmer_ids[:10])))
+                    # print(kmer_ids[:10])
+                    if j >= 0 and j + 1 <= seqlen:
+                        if positions[j] == pos: # The i-xth position has the same position in the sequence and is not a neighbor
+                            pass
+                        else:
+                            if positions[j] == pos - 1 and kmer.are_neighbors(current_kmer, kmer.id_to_kmer(kmer_ids[j], k)): # This is considered a left neighbor lexically
+                                left_neighbors.append( (kmer_ids[j], kmer_id) )
+                            else: # This is an exit condition
+                                tracking_left = False
+                    else:
+                        tracking_left = False
+                    j -= 1
+
+                """
+                No longer need to check for right neighbors lexically,
+                as all kmer edges are kmer_ids[i-1] -> kmer_id
+                """
+                # right_neighbors = []                    
+                # j = i + 1
+                # tracking_right = True
+                # while tracking_right is True:
+                #     if j < num_kmer_ids and seq_ids[j] == seq_id:
+                #         #print(f"{i} : {current_kmer} -> {kmer.id_to_kmer(kmer_ids[j], k)}")
+                #         if positions[j] == pos: # The i+xth position has the same position in the sequence and is not a neighbor
+                #             pass
+                #         elif positions[j] == pos + 1 and kmer.is_right_neighbor(current_kmer, kmer.id_to_kmer(kmer_ids[j], k)):
+                #             right_neighbors.append( (kmer_id, kmer_ids[j]) )
+                #         else:
+                #             tracking_right = False
+                #     else:
+                #         tracking_right = False
+                #     j += 1
+                # #This may be needed for debugging purposes
+                #ln = [tuple(map(lambda l: kmer.id_to_kmer(l, k), ln)) for ln in left_neighbors]
+                #rn = [tuple(map(lambda l: kmer.id_to_kmer(l, k), rn)) for rn in right_neighbors]
+                #print(f"{ln} | {i}:{current_kmer} ")
+                pairs_ = resolve_edges(i, kmer_id, current_kmer, seq_id, pos, seqlen, left_neighbors, k)
+                if len(pairs_) == 0:
+                    print("Are neighbors? {0}".format(list(map(lambda ln, ck: kmer.are_neighbors(ck, ln), left_neighbors))))
+                    print(left_neighbors)
+                    raise RuntimeError("UHoh")
+                if len(pairs_) > 0:
+                    final += [ (seq_id, positions[i-1], p[0], pos, kmer_id) for p in pairs_] # This
+    #print(final[:10])
+    return final
 
     
-    edge_list_length = n1.shape[0] #
-    edge_list = [ (int(n1[i]), int(n2[i]), {'weight': int(weights[i])}) for i in range(edge_list_length)]
-    G = nx.Graph()
-
-
-    G.add_nodes_from(range(N)) # Adds a list of k-mer ids as nodes to the graph
-    G.add_edges_from(edge_list)
-
-    print("Number of nodes: {0}".format(G.number_of_nodes()))
-    print("Number of edges: {0}".format(G.number_of_edges()))
-
-
-    
-def make_coo_graph(filename:str, k:int, quiet:bool=True, replace_with_none:bool=False):
+def make_edges_from_fasta(filename:str, k:int, quiet:bool=True, canonicalize:bool=True, replace_with_none:bool=False):
+    """
+    :param filename: Filepath to a fasta file to make edges from
+    :type filename: str
+    :param k:
+    :type k: int
+    :param quiet:
+    :type quiet: bool
+    :param canonicalize: Canonicalize k-mer ids lexicographically to be strand agnostic
+    :type canonicalize: bool
+    :param replace_with_none:
+    :type replace_with_none: bool
+    :returns: returns a 3-tuple - the edge list of 5-tuples (seq_id, pos1, kmerid1, pos2, kmerid2), file_metadata for .kdbg file, and the counts numpy.ndarray
+    :rtype: tuple
+    """
     if type(filename) is not str:
         raise TypeError("kmerdb.graph.make_coo_graph() expects a fasta/fastq sequence filepath as a str as its first positional_regument")
     elif type(k) is not int:
@@ -103,8 +253,11 @@ def make_coo_graph(filename:str, k:int, quiet:bool=True, replace_with_none:bool=
     """
     counts = np.zeros(N, dtype="uint64")
     kmer_ids = []
-    seq_lengths = []
-    pairs = []
+    seq_ids = []
+    positions = []
+    seq_lengths = {}
+    data = []
+    num_kmer_ids = 0
     for seq in parse.parse_sequence_file(filename, return_tuple=False):
         """
         Shred sequences and get a list of kmer ids in order. We do not count kmers at the moment.
@@ -115,60 +268,39 @@ def make_coo_graph(filename:str, k:int, quiet:bool=True, replace_with_none:bool=
         Store the results in numpy uint32 (~4B is the max size of unit32) if k<16 EXACTLY or uint64 otherwise
         FIXME: bonus_kmer_ids not included
         """
-        seq_lengths.append(len(seq.seq))
-        kmer_ids_, bonus_kmer_ids = kmer.shred(seq, k, replace_with_none=replace_with_none, quiet_iupac_warning=True)
-        kmer_ids.extend(kmer_ids_)
-        
-        for kmer_id in kmer_ids:
+        seqlen = len(seq.seq)
+        seq_lengths[seq.id] = seqlen 
+        kmer_ids_, seq_ids_, pos = kmer.shred(seq, k, replace_with_none=replace_with_none, canonicalize=canonicalize, quiet_iupac_warning=True)
+        data_ = make_edges_from_kmerids(kmer_ids_, pos, seqlen, seq.id, k)
+        num_kmer_ids += len(kmer_ids_)
+        if None in kmer_ids:
+            raise RuntimeError("Internal error: An invalid k-mer id was found within sequence '{0}'".format(seq.id))
+        # kmer_ids += kmer_ids_
+        # seq_ids += seq_ids_
+        # positions += pos
+        for kmer_id in kmer_ids_:
             if kmer_id is not None:
                 counts[kmer_id] += 1
-        if replace_with_none is False:
-            for kmer_id in bonus_kmer_ids:
-                if kmer_id is not None:
-                    counts[kmer_id] += 1
-        pass
-    num_kmer_ids = len(kmer_ids)
-    pairs_ = []
-    for i, k_id in enumerate(kmer_ids):
-        current_kmer = kmer.id_to_kmer(k_id, k)
-        #common_seq = current_kmer[1:-1]
+        data += data_
 
-        if i == 0:
-            left_neighbor_ids = kmer.left_neighbors(kmer.id_to_kmer(k_id, k))
-            pairs += list(zip( left_neighbor_ids, [k_id]*4))
-        elif i == num_kmer_ids - 1:
-            right_neighbor_ids = kmer.right_neighbors(kmer.id_to_kmer(k_id, k))
-            pairs += list(zip( [k_id]*4, right_neighbor_ids ))
-        else: # ALL OTHER KMERS
-            left_neighbor = kmer.id_to_kmer(kmer_ids[i-1], k)
-            right_neighbor = kmer.id_to_kmer(kmer_ids[i+1], k)
-            has_right_neighbor = kmer.is_right_neighbor(current_kmer, right_neighbor)
-            has_left_neighbor = kmer.is_left_neighbor(current_kmer, left_neighbor)
-            # FIXME: Convert these to kmer_ids instead of appending str pairs to pairs_
-            if has_left_neighbor is True and has_right_neighbor is True:
-                pairs.append( (kmer_ids[i-1], k_id) ) # left neighbor edge
-                pairs_.append( (left_neighbor, current_kmer ) )
-            elif has_right_neighbor is True and has_left_neighbor is False:
-                pairs.append( (k_id, kmer_ids[i+1]) )
-            elif has_right_neighbor is False and has_left_neighbor is True:
-                pairs.append( ( kmer_ids[i-1], k_id ) )
-            elif has_right_neighbor is False and has_left_neighbor is False:
-                raise ValueError("Internal error: Single k-mer found without obvious neighbors.")
 
-        """
-        FIXME: This produces all possible edges
-        The commented code will not cover read boundaries...
-        I actually DO want to not cover read boundaries, and then try to create as many Eulerian walks
-        Through a graph of the EDGES as NODES and the edges as valid adjacencies
+        
+    """
+    Break off functionality here. Need to pass a slice of k-mers and then resolve the k-mer group using conditionals.
+    The while-loop functionality should not be included here.
+    Instead, lists of k-mers passed to a function should resolve which neighbors get to be involved.
+    Completed 7/13/25
+    """
+    """
+    FIXME: This produces all possible edges
+    The commented code will not cover read boundaries...
+    I actually DO want to not cover read boundaries, and then try to create as many Eulerian walks
+    Through a graph of the EDGES as NODES and the edges as valid adjacencies
 
-        MAKE A kmer.are_neighbors function that creates all neighbors of two k-mers and verifies that they are neighbors.
+    MAKE A kmer.are_neighbors function that creates all neighbors of two k-mers and verifies that they are neighbors.
 
-        This is how we take an EDGE as a node, and then take the left k-mer and find its possible neighbors, and the right node and its possible neighbors, and create an EDGE between the right k-mer and its possible neighbors so long as there are neighbors left? I dunno..
-        """
-
-        """
-        [ (kmer_id, neighbor_kmer_id) , ... ]
-        """
+    This is how we take an EDGE as a node, and then take the left k-mer and find its possible neighbors, and the right node and its possible neighbors, and create an EDGE between the right k-mer and its possible neighbors so long as there are neighbors left? I dunno..
+    """
 
     md5, sha256 = util.checksum(filename)
 
@@ -177,11 +309,11 @@ def make_coo_graph(filename:str, k:int, quiet:bool=True, replace_with_none:bool=
     unique_kmers = int(np.count_nonzero(counts))
 
     
-    num_nullomers = N - unique_kmers
+    num_nullomers = N - unique_kmers if canonicalize is False else int((N/2) - unique_kmers)
     num_reads = len(seq_lengths)
-    max_read_length = max(seq_lengths)
-    min_read_length = min(seq_lengths)
-    avg_read_length = int(np.mean(np.array(seq_lengths)))
+    max_read_length = max(seq_lengths.values())
+    min_read_length = min(seq_lengths.values())
+    avg_read_length = int(np.mean(np.array(list(seq_lengths.values()))))
 
     file_metadata = {
         "filename": filename,
@@ -197,7 +329,30 @@ def make_coo_graph(filename:str, k:int, quiet:bool=True, replace_with_none:bool=
         "avg_read_length": avg_read_length
     }
 
-    return pairs, file_metadata, counts
+    return data, file_metadata, counts
+
+
+
+
+def resolve_edges(i:int, kmer_id:int, current_kmer:str, seq_id:str, pos:int, seqlen:int, left_neighbors:list, k:int): #kmer_ids:list, seq_ids:list, positions, seq_lengths:dict, left_neighbors:list, right_neighbors:list, k):
+    if pos + 1 > seqlen: # when this is the last position in the sequence (seq_len - 1), validate neighbor and return final kmer_edge
+        logger.error(f"kmer {i} '{current_kmer}' has {len(left_neighbors)} left neighbors and {len(right_neighbors)} right neighbors")
+        raise IndexError(f"Internal error: {i}th kmer {kmer_id}:{current_kmer} has {len(left_neighbors)} left neighbors: {left_neighbors} at position {pos} in a sequence of length {seqlen}")
+    # This sequence resolves
+    generic_error = ValueError("kmerdb.graph.make_edges_from_fasta() - difficulty deciphering sequence boundary and which neighboring kmers are considered valid neighbors")
+    
+    if len(left_neighbors) == 1 and kmer.are_neighbors(current_kmer, kmer.id_to_kmer(left_neighbors[0][0], k)):
+        assert left_neighbors[0] == (left_neighbors[0][0], kmer_id), "kmerdb.graph.resolve_edges() expects a pair of kmerids as the argument 'left_neighbors'"
+        return [ left_neighbors[0] ] # The left neighbor is valid, do retrospective edge list append
+    elif len(left_neighbors) > 1:
+        # This is the condition in which the current k-mer has multiple 'possible' left/right neighbors
+        return [ (ln[0], kmer_id) for ln in left_neighbors if kmer.are_neighbors(current_kmer, kmer.id_to_kmer(ln[0], k)) and ln[1] == kmer_id ]
+
+    else:
+        left_kmers = list(list(map(lambda km: (kmer.id_to_kmer(km[0], k), kmer.id_to_kmer(km[1], k)), left_neighbors)))
+        logger.error(f"index: {i}th kmer @ {pos} in {seq_id} : \n\n{left_kmers} -> {current_kmer}\n\n")
+        raise ValueError(f"Internal error: could not determine the category of k-mer relationships amongst the {i}th total k-mer '{current_kmer}' and its neighborhood amongst sequence '{seq_id}'")
+
 
 
 
@@ -356,14 +511,6 @@ class KDBGReader(bgzf.BgzfReader):
         :type fileobj: io.IOBase
         :param mode: read/write mode
         :type mode: str
-        :param n1_dtype: NumPy dtype
-        :type n1_dtype: str
-        :param n2_dtype: NumPy dtype
-        :type n2_dtype: str
-        :param weights_dtype: NumPy dtype
-        :type weights_dtype: str
-        :param sort: *unimplemented* - uh.. sort the .kdbg edges somehow
-        :type sort: bool
         :param slurp: Autoload the .kdbg file
         :type slurp: bool
         :raise TypeError: fileobj was not a file-type interface, inheriting from io.IOBase
@@ -427,7 +574,7 @@ class KDBGReader(bgzf.BgzfReader):
         if slurp is True:
             logger.info("Reading .kdbg contents into memory")
             self.slurp()
-
+        self.G = None
         if handle is not None:
             handle.close()
         if self._handle is not None:
@@ -546,7 +693,7 @@ Failed to validate YAML header.
         """
         # FIXME
         line = self.readline()
-        print("Line: {0}".format(line))
+        #print("Line: {0}".format(line))
         if line == '':
             return
         else:
@@ -558,29 +705,137 @@ Failed to validate YAML header.
         Autoload the .kdbg file into memory
         """
         #vmem = psutil.virtual_memory()
-        total_kmer_ids = []
-        kmer1 = []
-        kmer2 = []
+        seq_ids = []
+        pos1 = []
+        pos2 = []
+        kmerid1 = []
+        kmerid2 = []
+
         reading=True
         while reading is True:
             try:
                 line = next(self)
             except StopIteration as e:
                 logger.info("Finished loading .kdbg through slurp (on init)")
+                reading = False
                 break
             if line is None:
                 raise RuntimeError("Internal error: kmerdb.graph.KDBGReader.slurp errored on new line")
             try:
                 if line is not None:
-                    tkid, k1, k2 = list(map(int, line.split("\t")))
-                    total_kmer_ids.append(tkid)
-                    kmer1.append(k1)
-                    kmer2.append(k2)
+                    line = line.split("\t")
+                    if len(line) != 8:
+                        raise ValueError("kmerdb.graph.KDBGReader.slurp() expects .kdbg data to have 8 columns")
+                    i, seq_id, p1, id1, kmer1str, p2, id2, kmer2str = line
+                    try:
+                        if re.match(util.is_integer, i) is None:
+                            raise ValueError("kmerdb.graph.KDBGReader.slurp() expects the first column to be an integer value")
+                        elif re.match(util.is_integer, p1) is None:
+                            raise ValueError("kmerdb.graph.KDBGReader.slurp() expects the third column to be an integer value")
+                        elif re.match(util.is_integer, id1) is None:
+                            raise ValueError("kmerdb.graph.KDBGReader.slurp() expects the fourth column to be an integer value")
+                        elif re.match(util.is_integer, p2) is None:
+                            raise ValueError("kmerdb.graph.KDBGReader.slurp() expects the sixth column to be an integer value")
+                        elif re.match(util.is_integer, id2) is None:
+                            raise ValueError("kmerdb.graph.KDBGReader.slurp() expects the seventh column to be an integer value")
+                        
+                    except Exception as e:
+                        raise e
+                    seq_ids.append(seq_id)
+                    pos1.append(int(p1))
+                    pos2.append(int(p2))
+                    kmerid1.append(int(id1))
+                    kmerid2.append(int(id2))
             except StopIteration as e:
                 reading = False
-        self.total_kmer_ids   = np.array(total_kmer_ids, dtype="uint64")
-        self.kmer1_ids        = np.array(kmer1, dtype="uint64")
-        self.kmer2_ids        = np.array(kmer2, dtype="uint64")
+        self.seq_ids = seq_ids
+        self.pos1 = pos1 #np.array(pos1, dtype="uint64")
+        self.kmer_id1 = kmerid1 #np.arary(kmerid1, dtype="uint64")
+        self.pos2 = pos2 #np.array(pos2, dtype="uint64")
+        self.kmer_id2 = kmerid2 #np.arary(kmerid2, dtype="uint64")
+
 
         return
     
+    def as_networkx(self, seq_ids:list=None, pos1:list=None, kmer1:list=None, pos2:list=None, kmer2:list=None):
+        """
+        Creates a NetworkX representation of the graph.
+        
+        """
+        N = 4**self.k
+        if self.G is not None:
+            return self.G
+
+        
+        if seq_ids is None and pos1 is None and kmer1 is None and pos2 is None and kmer2 is None:
+            seq_ids = self.seq_ids
+            kmer1 = self.kmer_id1
+            kmer2 = self.kmer_id2
+            pos1 = self.pos1
+            pos2 = self.pos2
+            
+        num_edges = len(kmer1)
+        unique_kmers = list(set(kmer1))
+        temp = kmer1 + kmer2
+        edge_list = list(zip(kmer1, kmer2)) # Using a set here would be faster computationally, but would be challenging to get the seq_ids
+        edges = []
+        nodes = []
+
+        """
+        The edge list needs to be a reduced set. The edge list needs to have kmer_ids that are not redundant
+        The node list should also be a reduced set
+            
+        """
+            
+        for kmerid_ in temp:
+            try:
+                idx = kmer1.index(kmerid_)
+                pos = pos1[idx]
+            except ValueError as e:
+                idx = kmer2.index(kmerid_)
+                pos = pos2[idx]
+            nodes.append( (kmerid_, {"seq_id": seq_ids[idx], "pos": pos, "kmer": kmer.id_to_kmer(kmerid_, self.k) } ) )
+            
+
+        G = nx.DiGraph()
+        G.add_nodes_from(nodes)
+        G.add_edges_from(edge_list)
+
+        self.nodes = nodes
+        self.edges = edge_list
+            
+        print("Original number of kmers: {0}".format(len(seq_ids)))
+        print("Deduplicated number of nodes: {0}".format(len(nodes)))
+        print("Deduplicated number of edges: {0}".format(len(edge_list)))
+            
+            
+        logger.info(f"Number of nodes: {G.number_of_nodes()}")
+        logger.info(f"Number of edges: {G.number_of_edges()}")
+
+        return G
+        # Adjacency matrix
+        #rcm = list(cuthill_mckee_ordering(self.G, heuristic=_smallest_degree))
+        #self.A = nx.adjacency_matrix(self.G, nodelist=rcm)
+        return
+
+        
+    #def make_eulerian_circuit(self):
+
+    def _smallest_degree(self):
+        return min(self.G, key=self.G.degree)
+        
+    def write_dot(self, filepath:str):
+        from networkx.utils import cuthill_mckee_ordering
+        if type(filepath) is not str:
+            raise ValueError("kmerdb.graph.KDBGReader.write_dot() expects a filepath as a str")
+        #pg = nx.nx_pydot.to_pydot(G)
+        nx.write_dot(G, filepath)
+
+    # def read_dot(self, filepath):
+    #     if type(filepath) is not str and not (os.path.exists(filepath) and os.access(filepath, os.R_OK)):
+    #         raise ValueError("kmerdb.graph.KDBGReader.read_dot() expects a filepath as a str that points to a readable filepath on the filesystem")
+                 
+    #def write_sparse6(self, filepath:str):
+
+
+    
