@@ -59,11 +59,13 @@ def parse_sequence_file(seq_filepath:str, return_tuple:bool=True):
         raise ValueError("kmerdb.graph.parse_sequence_file() expects the filepath to be be readable on the filesystem")
 
     try:
-        logger.debug("Beginning to process sequence filepath '{0}".format(seq_filepath))
-        if util.is_gz_file is True:
-            seqhandle = gzip.open(seq_filepath, "rb")
+        logger.debug("Beginning to process sequence filepath '{0}'".format(seq_filepath))
+        if util.is_gz_file(seq_filepath) is True:
+            logger.debug("File '{0}' provided seems to be gzipped".format(seq_filepath))
+            seqhandle = gzip.open(seq_filepath, "rt", encoding="UTF-8")
         else:
-            seqhandle = open(seq_filepath, 'r')
+            logger.debug("File '{0}' provided seems to be uncompressed".format(seq_filepath))
+            seqhandle = open(seq_filepath, 'r', encoding="UTF-8")
         if util.is_fasta(seq_filepath) is True:
             parser = SeqIO.parse(seqhandle, "fasta")
         elif util.is_fastq(seq_filepath) is True:
@@ -85,7 +87,7 @@ def parse_sequence_file(seq_filepath:str, return_tuple:bool=True):
 
 
 
-def parsefile(filepath:str, k:int, replace_with_none:bool=True): 
+def parsefile(filepath:str, k:int, replace_with_none:bool=True, canonicalize:bool=True): 
     """Parse a single sequence file in blocks/chunks with multiprocessing support
 
     :param filepath: Path to a fasta or fastq file
@@ -125,22 +127,15 @@ def parsefile(filepath:str, k:int, replace_with_none:bool=True):
 
     for seq in parse_sequence_file(filepath, return_tuple=False):
         seq_id = seq.id
-        
-        kmer_ids, bonus_kmer_ids = kmer.shred(seq, k, replace_with_none=replace_with_none, quiet_iupac_warning=False)
-        seqlen = len(seq)
+        seqlen = len(seq)        
+        kmer_ids, seq_ids, pos = kmer.shred(seq, k, replace_with_none=replace_with_none, canonicalize=canonicalize, quiet_iupac_warning=False)
 
         for kmer_id in kmer_ids:
             if kmer_id is not None:
                 counts[kmer_id] += 1
-        if replace_with_none is False:
-            for kmer_id in bonus_kmer_ids:
-                if kmer_id is not None:
-                    counts[kmer_id] += 1
+                total_kmers += 1
         seq_lengths.append(seqlen)
-        total_kmers += len(kmer_ids) + len(bonus_kmer_ids)
 
-
-    #total_kmers = int(np.sum(counts)[0]),
     is_nullomer = np.where(counts == 0)
     nullomer_array = np.array(range(N), dtype="uint64")[is_nullomer]
     unique_kmers = int(np.count_nonzero(counts))
